# -*- coding: utf-8 -*-

from django.core.cache import cache
from django.shortcuts import render, redirect
from portailuser.models import PortailUser, Permission
from lib.access_auth import permission_needed
from django.db import transaction
from django.db.models.query_utils import Q

from portailuser.app_logger import AppLogger
from portailuser.admin_menu import ADMIN_MENU_CACHE_KEY

# Logs
_logger = AppLogger("gestion_permissions")


def cmp_permissions(permissions, utilisateur):
    liste = []
    perm_utilisateur = utilisateur.permissions.all()
    for i in permissions:
        liste.append((i.nom, i in perm_utilisateur))
    return liste


@permission_needed("gerer_permissions")
def liste(request):
    permissions = Permission.objects.filter(Q(ecole=request.ecole) | Q(ecole=None))
    utilisateurs = (
        PortailUser.objects.filter(Q(permissions__in=permissions) | Q(is_staff=True)).distinct().order_by("username")
    )

    for utilisateur in utilisateurs:
        utilisateur.cmp_permissions = cmp_permissions(permissions, utilisateur)

    c = {
        "permissions": permissions,
        "utilisateurs": utilisateurs,
    }

    return render(request, "gestion/permissions/liste.html", c)


@permission_needed("gerer_permissions")
@transaction.atomic
def editer(request, uid):
    utilisateur = PortailUser.objects.get_from_ldap_or_404(uid)
    permissions = Permission.objects.filter(Q(ecole=request.ecole) | Q(ecole=None))
    perm_act = utilisateur.permissions.all()

    for p in permissions:
        p.active = p in perm_act

    if request.method == "POST":
        for p in permissions:
            if f"perm_{p.pk}" in request.POST and not p.active:
                utilisateur.permissions.add(p)
                _logger.log(
                    request.user,
                    "A ajouté la permission ({} - {}) à « {} »",
                    p.desc,
                    p.ecole,
                    uid,
                )
            elif f"perm_{p.pk}" not in request.POST and p.active:
                utilisateur.permissions.remove(p)
                _logger.log(
                    request.user,
                    "A enlevé la permission ({} - {}) à « {} »",
                    p.desc,
                    p.ecole,
                    uid,
                )

        cache.delete(ADMIN_MENU_CACHE_KEY.format(utilisateur.id, request.ecole.id))

        return redirect("gestion:permissions:liste")

    c = {
        "utilisateur": utilisateur,
        "permissions": permissions,
    }

    return render(request, "gestion/permissions/editer.html", c)
